//Created by Neo on 19th Aug 2020
import 'dart:collection';
import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'dart:io';

import 'package:nui_core/nui_core.dart';

typedef DateFiltering = bool Function(DateTime date);
typedef DateSelectListener = void Function(DateTime date);
typedef TimeSelectListener = void Function(TimeOfDay time);

String dateToString(DateTime date, {String format : NUIDTUtil.FORMAT_yyyy_MM_dd_HH_mm_ss}){
  return NUIDTUtil.dateToString(date, format: format);
}

DateTime stringToDate(String dateString, {String dateFormat}){
  return NUIDTUtil.stringToDate(dateString, dateFormat: dateFormat);
}

String formatTime(BuildContext context, TimeOfDay timeOfDay, {bool use24HourFormat: true}){
  return NUIDTUtil.formatTime(context, timeOfDay, use24HourFormat: use24HourFormat ?? true);
}

DateTime startOfDay(){
  return NUIDTUtil.startOfDay();
}

DateTime endOfDay(){
  return NUIDTUtil.endOfDay();
}

class NUIDTUtil{
  NUIDTUtil._builder();
  static const FORMAT_yyyy_MM_dd = "yyyy-MM-dd";
  static const FORMAT_hh_mm_dd_MMM_yyyy = "HH:mm, dd MMM yyyy";
  static const FORMAT_hh_mm_a_dd_MMM_yyyy = "hh:mm a, dd MMM yyyy";
  static const FORMAT_dd_MMM_yyyy_HH_mm = "dd MMM yyyy, HH:mm";
  static const FORMAT_dd_MMM_yyyy_hh_mm_a = "dd MMM yyyy, hh:mm a";
  static const FORMAT_dd_MMM_yyyy = "dd MMM yyyy";
  static const FORMAT_d_MMM_yyyy = "d MMM yyyy";
  static const FORMAT_dd_MM_yyyy = "dd/MM/yyyy";
  static const FORMAT_dd_MM_yyyy_dash = "dd-MM-yyyy";
  static const FORMAT_dd_MMM = "dd MMM";
  static const FORMAT_d_MMM = "d MMM";
  static const FORMAT_yyyy_MM_dd_HH_mm_ss = "yyyy-MM-dd HH:mm:ss";
  static const FORMAT_yyyyMMdd_HHmmss = "yyyyMMdd_HHmmss";
  static const FORMAT_yyyyMMdd_HHmmA = "yyyy/MM/dd hh:mm a";
  static const FORMAT_MMM_d_yyyy = "MMM d, yyyy";
  static const FORMAT_hh_mm_a = "hh:mm a";
  static const FORMAT_hh_mm_12 = "hh:mm";
  static const FORMAT_HH_mm_24 = "HH:mm";
  static const FORMAT_EEE_MMM_d_yyy = "EEE, MMM d yyyy";
  static const FORMAT_EEE_DD_MMM_yyyy = "EEE, dd MMM yyyy";
  static const FORMAT_EEEE_MMM_d_yyy = "EEEE, dd MMM yyyy";
  static const FORMAT_EEEE_DD_MMM_yyyy = "EEEE, dd MMM yyyy";
  static const FORMAT_yyyy_MM_dd_T_HH_mm_ss_Z = "yyyy-MM-dd'T'HH:mm:ss Z";

  static String formatTime(BuildContext context, TimeOfDay timeOfDay, {bool use24HourFormat: true}){
    final MaterialLocalizations localizations = MaterialLocalizations.of(context);
    return localizations.formatTimeOfDay(timeOfDay, alwaysUse24HourFormat: use24HourFormat ?? true);
  }

  static DateTime endOfDay(){
    final now = DateTime.now();
    return DateTime(now.year, now.month, now.day, 23, 59, 59);
  }

  static DateTime startOfDay(){
    final now = DateTime.now();
    return DateTime(now.year, now.month, now.day, 0, 0, 0);
  }

  static DateTime firstDayOfMonth({int month, int year}) {
    DateTime cal = DateTime.now();
    final firstDayOfMonth = DateTime.utc(year == null ? cal.year : year, month == null ? cal.month : month, 1);
    return firstDayOfMonth;
  }

  static DateTime lastDayOfMonth({int month, int year}) {
    DateTime now = new DateTime.now();
    int targetMonth = month == null ? now.month : month;
    int targetYear = year == null ? now.year : year;
    if(targetMonth == 12){
      targetYear = targetYear + 1;
      targetMonth = 1;
    }
    else{
      targetMonth = targetMonth + 1;
    }
    DateTime lastDayOfMonth = new DateTime(targetYear, targetMonth, 0);
    return lastDayOfMonth;
  }

  static bool isSameDay(DateTime date1, DateTime date2){
    bool isSameDay = date1.day == date2.day;
    bool isSameMonth = date1.month == date2.month;
    bool isSameYear = date1.year == date2.year;
    return isSameDay && isSameMonth && isSameYear;
  }

  static bool isToday(DateTime date) {
    return isSameDay(DateTime.now(), date);
  }

  static bool isWithin24Hrs(DateTime date){
    Duration diff = DateTime.now().difference(date);
    final diffHours = diff.inHours < 0 ? diff.inHours*-1 : diff.inHours;
    return diffHours <= 24;
  }

  static bool isThisWeek(DateTime date){
    Duration diff = DateTime.now().difference(date);
    final diffDays = diff.inDays < 0 ? diff.inDays*-1 : diff.inDays;
    return diffDays <= 7;
  }

  static bool isThisMonth(DateTime date){
    final curCal = DateTime.now();
    bool isSameMonth = curCal.month == date.month;
    bool isSameYear = curCal.year == date.year;
    return isSameMonth && isSameYear;
  }

  static String dateToString(DateTime date, {String format : "yyyy-MM-dd"}){
    try {
      return DateFormat(format).format(date);
    }catch(e){
      return dateToString(date);
    }
  }

  static DateTime stringToDate(String dateString, {String dateFormat}){
    DateTime dateTime;
    try {
      dateTime = DateTime.parse(dateString);
    }catch(e){
      logNUI("NUIDTUtil", "Failed to parse date directly with error : $e");
    }

    if(dateTime == null){
      try{
        if(!isNullOrEmpty(dateFormat)){
          dateTime = DateFormat(dateFormat).parse(dateString);
        }
        else{
          dateTime = DateFormat(detectDateFormat(dateString)).parse(dateString);
        }
      }catch(e){
        logNUI("NUIDTUtil", "Failed to parse date dynamicly with error : $e");
      }
    }
    return dateTime;
  }

  static int getTimeDiffInLong(DateTime previousDate, {DateTime currentDate, String dateFormat: FORMAT_dd_MMM_yyyy}){
    DateTime targetDate = currentDate == null ? DateTime.now() : currentDate;
    return targetDate.millisecondsSinceEpoch - previousDate.millisecondsSinceEpoch;
  }

  static String getDateTimeDiffInShort(DateTime previousTime, {DateTime currentDate}){
    final targetDate = currentDate == null ? DateTime.now() : currentDate;
    final dateDiff = targetDate.difference(previousTime);

    final day = dateDiff.inDays < 0 ? dateDiff.inDays*-1 : dateDiff.inDays;

    if(day > 31){
//      if(day > 365){
        return dateToString(previousTime, format: FORMAT_d_MMM_yyyy);
//      }
//      return dateToString(previousTime, format: FORMAT_d_MMM);
    }

    if(day > 0){
      return "${day.toInt()}d";
    }

    final hour = dateDiff.inHours < 0 ? dateDiff.inHours*-1 : dateDiff.inHours;
    if(hour > 0){
      return "${hour.toInt()}h";
    }

    final minute = dateDiff.inMinutes < 0 ? dateDiff.inMinutes*-1 : dateDiff.inMinutes;
    if(minute > 0){
      return "${minute.toInt()}m";
    }

    final second = dateDiff.inSeconds < 0 ? dateDiff.inSeconds*-1 : dateDiff.inSeconds;
    if(second.toInt() < 1){
      return "Now";
    }
    if(second.toInt() == 1){
      return "1s";
    }
    return "${second.toInt()}s";
  }

  static String getDateDiffAsString(DateTime previousTime, {DateTime currentDate, String dateFormat: FORMAT_EEE_MMM_d_yyy}){
    final targetDate = currentDate ?? DateTime.now();
    final dateDiff = targetDate.difference(previousTime);

    final day = dateDiff.inDays < 0 ? dateDiff.inDays*-1 : dateDiff.inDays;

    if(day > 31){
//      if(day > 365){
        return dateToString(previousTime, format: FORMAT_d_MMM_yyyy);
//      }
//      return dateToString(previousTime, format: FORMAT_d_MMM);
    }

    if(day > 0){
      if(day.toInt() == 1) {
        return "${day.toInt()} day ago";
      }
      if(day.toInt() > 6){
        return dateToString(previousTime, format: dateFormat ?? FORMAT_EEE_MMM_d_yyy);
      }
      return "${day.toInt()} days ago";
    }

    final hour = dateDiff.inHours < 0 ? dateDiff.inHours*-1 : dateDiff.inHours;
    if(hour > 0){
      if(hour.toInt() == 1){
        return "1 hr ago";
      }
      return "${hour.toInt()} hrs ago";
    }

    final minute = dateDiff.inMinutes < 0 ? dateDiff.inMinutes*-1 : dateDiff.inMinutes;
    if(minute > 0){
      if(minute.toInt() == 1){
        return "1 min ago";
      }
      return "${minute.toInt()} mins ago";
    }

    final second = dateDiff.inSeconds < 0 ? dateDiff.inSeconds*-1 : dateDiff.inSeconds;
    if(second.toInt() < 1){
      return "Just now";
    }
    if(second.toInt() == 1){
      return "1 sec ago";
    }
    return "${second.toInt()} secs ago";
  }

  static String getDateTimeDiffAsString(DateTime previousTime, {DateTime currentDate}){
    final targetDate = currentDate == null ? DateTime.now() : currentDate;
    final dateDiff = targetDate.difference(previousTime);

    final day = dateDiff.inDays < 0 ? dateDiff.inDays*-1 : dateDiff.inDays;
    if(day > 0){
      if(day.toInt() == 1) {
        return "1 day ago at ${dateToString(previousTime, format: FORMAT_hh_mm_a)}";
      }
      if(day.toInt() > 6){
       return "${dateToString(previousTime, format: FORMAT_EEE_MMM_d_yyy)}, ${dateToString(previousTime, format: FORMAT_hh_mm_a)}";
      }
      return "${day.toInt()} days ago at ${dateToString(previousTime, format: FORMAT_hh_mm_a)}";
    }

    final hour = dateDiff.inHours < 0 ? dateDiff.inHours*-1 : dateDiff.inHours;
    if(hour > 0){
      if(hour.toInt() == 1){
        return "1 hr ago";
      }
      return "${hour.toInt()} hrs ago";
    }

    final minute = dateDiff.inMinutes < 0 ? dateDiff.inMinutes*-1 : dateDiff.inMinutes;
    if(minute > 0){
      if(minute.toInt() == 1){
        return "1 min ago";
      }
      return "${minute.toInt()} mins ago";
    }

    final second = dateDiff.inSeconds < 0 ? dateDiff.inSeconds*-1 : dateDiff.inSeconds;
    if(second.toInt() < 1){
      return "Just now";
    }
    if(second.toInt() == 1){
      return "1 sec ago";
    }
    return "${second.toInt()} secs ago";
  }

  static getDateTimeDiffShort(DateTime previousTime, {DateTime currentDate}){
    final targetDate = currentDate == null ? DateTime.now() : currentDate;
    final dateDiff = targetDate.difference(previousTime);

    final day = dateDiff.inDays < 0 ? dateDiff.inDays*-1 : dateDiff.inDays;
    if(day > 0){
      return "${day.toInt()}d";
    }

    final hour = dateDiff.inHours < 0 ? dateDiff.inHours*-1 : dateDiff.inHours;
    if(hour > 0){
      return "${hour.toInt()}hr";
    }

    final minute = dateDiff.inMinutes < 0 ? dateDiff.inMinutes*-1 : dateDiff.inMinutes;
    if(minute > 0){
      return "${minute.toInt()}min";
    }

    final second = dateDiff.inSeconds < 0 ? dateDiff.inSeconds*-1 : dateDiff.inSeconds;
    if(second.toInt() < 1){
      return "Now";
    }

    return "${second.toInt()}sec";
  }

  static String detectDateFormat(String dateString){

    HashMap matchHash = HashMap<String, String>();
    matchHash["xxxxxxxx_xxxxxx"] = "yyyyMMdd_HHmmss";
    matchHash["xxxxxxxx xxxxxx"] = "yyyyMMdd HHmmss";
    matchHash["xxxx-xx-xx xx:xx:xx"] = "yyyy-MM-dd HH:mm:ss";
    matchHash["xxxx-xx-xx xx:xx"] = "yyyy-MM-dd HH:mm";
    matchHash["xxxx/xx/xx xx:xx:xx"] = "yyyy/MM/dd HH:mm:ss";
    matchHash["xxxx/xx/xx xx:xx"] = "yyyy/MM/dd HH:mm";
    matchHash["xx-xx-xxxx xx:xx:xx"] = "dd-MM-yyyy HH:mm:ss";
    matchHash["xx-xx-xxxx xx:xx"] = "dd-MM-yyyy HH:mm";
    matchHash["xx/xx/xxxx xx:xx:xx"] = "dd/MM/yyyy HH:mm:ss";
    matchHash["xx/xx/xxxx xx:xx"] = "dd/MM/yyyy HH:mm";
    matchHash["xxxx-xx-xx"] = "yyyy-MM-dd";
    matchHash["xxxx/xx/xx"] = "yyyy/MM/dd";
    matchHash["xx-xx-xxxx"] = "dd-MM-yyyy";
    matchHash["xx/xx/xxxx"] = "dd/MM/yyyy";
    matchHash["xx:xx:xx xxxx-xx-xx"] = "HH:mm:ss yyyy-MM-dd";
    matchHash["xx:xx:xx xxxx/xx/xx"] = "HH:mm:ss yyyy/MM/dd";
    matchHash["xx:xx:xx xxxx-xx-xx"] = "HH:mm:ss yyyy-MM-dd";
    matchHash["xx:xx:xx xxxx/xx/xx"] = "HH:mm:ss yyyy/MM/dd";
    matchHash["xxxx-xx-xx xx:xx"] = "yyyy-MM-dd HH:mm";
    matchHash["xxxx/xx/xx xx:xx"] = "yyyy/MM/dd HH:mm";
    matchHash["xx:xx"] = "HH:mm";
    matchHash["xx:xx xx"] = "hh:mm a";
    matchHash["xx xxx xxxx"] = "dd MMM yyyy";
    matchHash["x xxx xxxx"] = "d MMM yyyy";
    matchHash["xxxx xxx xx"] = "yyyy MMM dd";
    matchHash["xxxx xxx x"] = "yyyy MMM d";

    final regex = RegExp("[0-9]");
    final regexAlphabets = RegExp("[a-z]");
    final regexAlphabetsCaps = RegExp("[A-Z]");
    var numberRemovedString = dateString.replaceAll(regex,"x");
    numberRemovedString = numberRemovedString.replaceAll(regexAlphabets, "x");
    numberRemovedString = numberRemovedString.replaceAll(regexAlphabetsCaps, "x");
    final targetFormat = matchHash[numberRemovedString];

    if(targetFormat != null){
    return targetFormat;
    }

    return "yyyy-MM-dd HH:mm:ss";
  }

  static List<DateTime> getDatesBetween(DateTime date1, DateTime date2, {bool inclusive: true}){
    var smallerDate = date1.millisecondsSinceEpoch < date2.millisecondsSinceEpoch ? date1 : date2;
    var largerDate = date2.millisecondsSinceEpoch > date1.millisecondsSinceEpoch ? date2 : date1;

    final dates = List<DateTime>();

    while (!smallerDate.isAfter(largerDate)) {
      dates.add(smallerDate);
      smallerDate = DateTime.fromMillisecondsSinceEpoch(smallerDate.millisecondsSinceEpoch + Duration(days: 1).inMilliseconds);
    }

    if(!inclusive){
      if(dates.length > 0) {
        dates.removeAt(0);
      }
      if(dates.length > 0) {
        dates.removeLast();
      }
    }
    return dates;
  }

  static List<String> getDatesBetweenInString(DateTime date1, DateTime date2, {String dateFormat : FORMAT_yyyy_MM_dd, bool inclusive : true}) {
    final datesBetween = getDatesBetween(date1, date2, inclusive: inclusive);

    List<String> datesList = new List<String>();

    for(DateTime d in datesBetween){
      datesList.add(dateToString(d, format: dateFormat));
    }

    return datesList;
  }

  static Future<TimeOfDay> timeSelect(
      BuildContext context, {
        TimeOfDay initialTime,
        TimePickerEntryMode initialEntryMode: TimePickerEntryMode.dial,
        String helpText: "Select time",
        String cancelText: "Cancel",
        String confirmText:"Confirm",
        TimeSelectListener selectListener}) async{

    Future<TimeOfDay> timeSelected;
    final now = DateTime.now();
    if(Platform.isIOS){
      final datePicker = CupertinoDatePicker(
        onDateTimeChanged: (date) async{
          if(selectListener != null){
            selectListener(TimeOfDay.fromDateTime(date));
          }
          timeSelected = NUIAsync.delayForResult(Duration(milliseconds: 100), () async{
            final timeOfDay = TimeOfDay.fromDateTime(date);
            return timeOfDay;
          });
        },
        initialDateTime: DateTime(now.year, now.month, now.day, initialTime?.hour ?? now.hour, initialTime?.minute ?? now.minute, now.second),
        mode: CupertinoDatePickerMode.time,
      );
      showNUIBottomSheet(context: context, content: (context){
        return Container(
          height: screenHeight(context)/3,
          color: NUIColors.NUIWhite,
          child: datePicker,
        );
      });
    }
    else {
      timeSelected = showTimePicker(
          context: context,
          initialTime: initialTime ?? TimeOfDay.now(),
          initialEntryMode: initialEntryMode,
          helpText: helpText,
          cancelText: cancelText,
          confirmText: confirmText
      );
    }

    final timeResult = await timeSelected;
    if (selectListener != null) {
      selectListener(timeResult);
    }
    return timeResult;
  }

  static Future<DateTime> dateSelect(
      BuildContext context, {
        DateTime selectedDate,
        DateTime firstDate,
        DateTime lastDate,
        DatePickerEntryMode initialEntryMode: DatePickerEntryMode.calendar,
        DatePickerMode initialDatePickerMode: DatePickerMode.day,
        DateFiltering dateFiltering,
        String helpText: "Select date",
        String cancelText: "Cancel",
        String confirmText:"Confirm",
        String invalidText:"Enter a valid date",
        String fieldLabelText:"Date",
        String fieldHintText: "yyyy/mm/dd",
        DateSelectListener selectListener}) async{


    log("NUIDTUtil -> Showing date picker");
    Future<DateTime> dateTime;

    DateTime initialDate;
    final mySelectedDate = selectedDate ?? DateTime.now();
    if(mySelectedDate != null){
      if(lastDate != null && mySelectedDate.isAfter(lastDate)){
        initialDate = lastDate;
      }
      else if(firstDate != null && firstDate.isAfter(mySelectedDate)){
        initialDate = firstDate;
      }
      else{
        initialDate = mySelectedDate;
      }
    }


    if(Platform.isIOS){
      final datePicker = CupertinoDatePicker(
          onDateTimeChanged: (date) async{
            if(selectListener != null){
              selectListener(date);
            }
            dateTime = NUIAsync.delayForResult(Duration(milliseconds: 100), () async{
              return date;
            });
          },
          initialDateTime: mySelectedDate ?? DateTime.now(),
          minimumDate: firstDate ?? DateTime(DateTime.now().year - 100),
          maximumDate: lastDate ?? DateTime(DateTime.now().year + 100),
          mode: CupertinoDatePickerMode.date,
      );
      showNUIBottomSheet(context: context, content: (context){
        return Container(
          height: screenHeight(context)/3,
          color: NUIColors.NUIWhite,
          child: datePicker,
        );
      });
    }
    else {
      dateTime = showDatePicker(
          context: context,
          initialDate: initialDate ?? mySelectedDate ?? DateTime.now(),
          firstDate: firstDate ?? DateTime(DateTime.now().year - 100),
          lastDate: lastDate ?? DateTime(DateTime.now().year + 100),
          initialEntryMode: initialEntryMode,
          initialDatePickerMode: initialDatePickerMode,
          selectableDayPredicate: dateFiltering,
          helpText: helpText,
          confirmText: confirmText,
          cancelText: cancelText,
          errorFormatText: "Enter a valid date",
          errorInvalidText: invalidText,
          fieldLabelText: fieldLabelText,
          fieldHintText: fieldHintText,
      );
    }

    final dateResult = await dateTime;
    if(selectListener != null){
      selectListener(dateResult);
    }
    return dateResult;
  }

}