import 'package:flutter/cupertino.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_core/ui/component/listview/nui_list_item_repo.dart';
import 'package:nui_core/ui/component/listview/nui_listview_bloc.dart';
import 'nui_list_interface.dart';
import 'nui_listview_builder.dart';
import 'nui_listview_functions.dart';

class NUIAnimatedHeaderList<T, C> extends NUIAnimatedListInterface<T, C> {
  NUIAnimatedHeaderList({
    this.header,
    @required NUIListViewBloc<T, C> bloc,
    @required NUIListItemRepository repo,
    @required List<T> initialList,
    @required OnListItemClickListener clickListener,
    bool shrinkWrap: false,
    double startDivider,
    double endDivider,
    double middleDivider,
    ListFixedItemGetter fixedFirstItem,
    ListFixedItemGetter fixedLastItem,
    Axis orientation: Axis.vertical,
    EdgeInsets padding,
    bool enablePagination: false,
    bool enableSwipeRefresh: false,
    int minimumPaginateCount: 5,
    NUILoadingListItem skeletalInflater,
    NUILoadingListItem paginationInflater,
    ListShadowInflater shadowInflater,
    ListPlaceHolderInflater placeHolder,
    ListErrorHolderInflater errorHolder,
    NUIShaderMaskType shaderType,
    int skeletalCount: 1,
    Duration itemAnimationDuration,
    ScrollPhysics scrollPhysics,
    Map<String, dynamic> checkedMap,
    Map<String, dynamic> configMap,
    NUIListController controller,
    NUIListScrollListener scrollListener,
    this.withOverlapsAbsorber: false,
    NUIListItemAnimation itemAnimation: NUIListItemAnimation.SCALE
  }) : super(
      repo: repo,
      initialList: initialList,
      orientation: orientation,
      padding: padding,
      shrinkWrap: shrinkWrap,
      startDivider: startDivider,
      endDivider: endDivider,
      middleDivider: middleDivider,
      fixedFirstItem: fixedFirstItem,
      fixedLastItem: fixedLastItem,
      checkedMap: checkedMap,
      configMap: configMap,
      clickListener: clickListener,
      skeletalCount: skeletalCount,
      errorHolder: errorHolder,
      placeHolder: placeHolder,
      skeletalInflater: skeletalInflater,
      paginationInflater: paginationInflater,
      shaderType: shaderType,
      scrollPhysics: scrollPhysics,
      enablePagination: enablePagination,
      scrollListener: scrollListener,
      itemAnimationDuration: itemAnimationDuration,
      enableSwipeRefresh: enableSwipeRefresh,
      minimumPaginateCount: minimumPaginateCount,
      bloc: bloc,
      controller: controller,
      itemAnimation : itemAnimation ?? NUIListItemAnimation.SCALE
  );
  final Widget header;
  final bool withOverlapsAbsorber;

  @override
  _NUIAnimatedHeaderListState<T, C> createState() => _NUIAnimatedHeaderListState<T, C>();
}

class _NUIAnimatedHeaderListState<T, C> extends NUIAnimatedListState<T, C, NUIAnimatedHeaderList<T, C>> {
  final String id = randomUUID();

  @override
  Widget getListView(int initialCount) {
    logNUI("NUIAnimatedHeaderList", "Building animated header list view");

    Widget sliverList;
    if(!curModel.isLoading() && curModel.success == false && isNullOrEmpty(curModel.data) && widget.errorHolder != null){
      sliverList = SliverToBoxAdapter(child: widget.errorHolder(curModel));
    }
    else if(!curModel.isLoading() && isNullOrEmpty(curModel.data) && widget.placeHolder != null){
      sliverList = SliverToBoxAdapter(child: widget.placeHolder());
    }
    else {
      sliverList = SliverPadding(
          padding: widget.padding != null ? widget.padding : EdgeInsets.all(0),
          sliver: SliverAnimatedList(
            key: sliverListKey,
            initialItemCount: initialCount,
            itemBuilder: (context, position, animation){
              if(position < items.length) {
                return animateItem(context, position, animation);
              }
              return null;
            }
          )
      );
    }

    final slivers = List<Widget>();
    if(widget.header != null) {
      slivers.add(SliverToBoxAdapter(child: widget.header,),);
    }
    slivers.add(sliverList);

    Widget scrollView;
    if(widget.withOverlapsAbsorber == true) {
      logNUI("NUIAnimatedHeaderList", "With overlaps absorber enabled, wrapping list with SliverOverlapInjector");
      scrollView = Builder(
        builder: (BuildContext context) {
          final innerSlivers = List<Widget>();
          innerSlivers.add(SliverOverlapInjector(
            handle: NestedScrollView.sliverOverlapAbsorberHandleFor(context),
          ));
          innerSlivers.addAll(slivers);

          return CustomScrollView(
            physics: widget.scrollPhysics ?? NUICustomScrollPhysics(),
            scrollDirection: orientation(),
            shrinkWrap: widget.shrinkWrap ?? false,
            key: PageStorageKey<String>(id),
            slivers: innerSlivers
          );
        },
      );
    }
    else {
      scrollView = CustomScrollView(
        physics: widget.scrollPhysics ?? NUICustomScrollPhysics(),
        scrollDirection: orientation(),
        shrinkWrap: widget.shrinkWrap ?? false,
//        controller: scrollController(),
        slivers: slivers
      );
    }
    return scrollView;
  }

}



