import 'package:flutter/cupertino.dart';
import 'package:nui_core/bloc/nui_bloc.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/model/list_result.dart';
import 'package:nui_core/ui/component/listview/nui_listview_functions.dart';

import 'nui_list_models.dart';

typedef NUIListViewLoader<T, C> = Future<ListResult<T, C>> Function(List<T> current, @required bool isFirst, bool append, C cursor);

class NUIListViewBloc<T, C> extends NUIBloc<NUIListViewModel<T, C>>{
  NUIListViewBloc({this.callback, this.callbackMethod});
  NUIListViewCallback<T, C> callback;
  NUIListViewLoader<T, C> callbackMethod;

  void load(bool isFirst, List<T> current, bool append, bool skeletal, C cursor, {Function onLoadComplete}) async{
    ListResult<T, C> newModel;
    if(callback != null){
      newModel = await callback.load(isFirst: isFirst, current: current, append: append, cursor: cursor);
    }
    else{
      newModel = await callbackMethod(current, isFirst, append, cursor);
    }

    List<T> newList = List<T>();
    if(append && isFirst != true){
      newList.addAll(current);
    }
    if(!isNullOrEmpty(newModel.data)) {
      newList.addAll(newModel.data);
    }
    if(onLoadComplete != null) onLoadComplete();
    notify(NUIListViewModel<T, C>(skeletalLoading: false, paginationLoading:false, swipeRefreshing:false, reachedEnd: newModel.reachedEnd, appending: append && isFirst != true, success: newModel.success, message: newModel.message, errorCode: newModel.errorCode, data: newList, newData: newModel.data, cursor: newModel.cursor, updateCheckedMap: newModel.updateCheckedMap, updateConfigMap: newModel.updateConfigMap));
  }

  void startSkeletalLoading({Function onLoadComplete}){
    notify(NUIListViewModel<T, C>(skeletalLoading: true, paginationLoading:false, success: false, message: "Skeletal loading", data: null));
    load(false, null, false, true, null, onLoadComplete: onLoadComplete);
  }

  void startPaginationLoading(bool isFirst, List<T> current, dynamic cursor, {Function onLoadComplete}){
    notify(NUIListViewModel<T, C>(skeletalLoading: false, paginationLoading:true, success: false, message: "Pagination loading", data: current, cursor: cursor));
    load(isFirst, current, true, false, cursor, onLoadComplete: onLoadComplete);
  }

  void startSwipeRefreshLoading(List<T> current, {Function onLoadComplete}){
    load(false, current, false, false, null, onLoadComplete: onLoadComplete);
  }

}

abstract class NUIListViewCallback<T, C>{
  ListResult<T, C> lastResult;
  Future<ListResult<T, C>> load({List<T> current, @required bool isFirst, bool append: false, C cursor});
}

class NUIListActionController{
  NUIListActionController(this._parentHandler, this.model);
  NUIListViewParentHandler _parentHandler;
  NUIListItemModel model;

  void onItemClick(dynamic data, String action){
    _parentHandler.onItemClick(data, model.position, action);
  }

  void updateCheckedData(UpdateListMap task){
    _parentHandler.updateCheckedMap((map) {task(map);});
  }

  void updateConfigData(UpdateListMap task){
    _parentHandler.updateConfigMap((map) {task(map);});
  }

  dynamic getCheckedData(String key){
    return model.checkedMap[key];
  }

  dynamic getConfigData(String key){
    return model.configMap[key];
  }
}

class NUIListViewParentHandler{
  NUIListViewParentHandler(this._clickListener, this._checkedMap, this._configMap, this._checkedUpdated, this._configUpdated);
  OnListItemClickListener _clickListener;
  Map<String, dynamic> _checkedMap;
  Map<String, dynamic> _configMap;
  Function _checkedUpdated;
  Function _configUpdated;

  void onItemClick(dynamic data, int position, String action){
    _clickListener(data, position, action);
  }
  void updateCheckedMap(UpdateListMap task){
    task(_checkedMap);
    _checkedUpdated();
  }
  void updateConfigMap(UpdateListMap task){
    task(_configMap);
    _configUpdated();
  }
}