import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:nui_core/bloc/nui_bloc.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/ui/component/listview/nui_list_item_repo.dart';
import 'package:nui_core/ui/component/listview/nui_listview_bloc.dart';
import 'nui_list_models.dart';
import 'nui_listview_functions.dart';
import 'package:pull_to_refresh/pull_to_refresh.dart';
import 'nui_listview_widgets.dart';

@Deprecated("Use NUIAnimatedList widget for more feature")
class NUIListView<T, C> extends StatefulWidget {
  NUIListView({@required this.bloc, @required this.repo, @required this.initialList, @required this.clickListener, this.shrinkWrap: false, this.startDivider, this.endDivider, this.middleDivider, this.fixedFirstItem, this.fixedLastItem, this.withItemCount:true, this.cacheOffset, this.orientation: Axis.vertical, this.padding, this.enablePagination: false, this.enableSwipeRefresh:false, this.minimumPaginateCount: 5, this.skeletalInflater, this.paginationInflater, this.placeHolder, this.errorHolder, this.skeletalCount:1, this.scrollPhysics, this.checkedMap, this.configMap});
  final NUIListItemRepository repo;
  final List<T> initialList;
  final bool withItemCount;
  final Axis orientation;
  final EdgeInsets padding;
  final double cacheOffset;

  final bool shrinkWrap;

  final double startDivider;
  final double middleDivider;
  final double endDivider;

  final ListFixedItemGetter fixedFirstItem;
  final ListFixedItemGetter fixedLastItem;

  final Map<String, dynamic> checkedMap;
  final Map<String, dynamic> configMap;

  //Click callback
  final OnListItemClickListener clickListener;

  //Loader widgets
  final int skeletalCount;

  //Widget functions
  final ListErrorHolderInflater<T, C> errorHolder;
  final ListPlaceHolderInflater placeHolder;
  final ListLoadingInflater skeletalInflater;
  final ListLoadingInflater paginationInflater;

  final ScrollPhysics scrollPhysics;

  final bool enablePagination;
  final int minimumPaginateCount;
  final bool enableSwipeRefresh;

  final NUIListViewBloc<T, C> bloc;
  @override
  _NUIListViewState<T, C> createState() => _NUIListViewState<T, C>();
}

class NUICustomScrollPhysics extends ClampingScrollPhysics{
  const NUICustomScrollPhysics({ScrollPhysics parent}): super(parent: parent);

  @override
  double get minFlingVelocity => super.minFlingVelocity * 0.4;

  @override
  double get maxFlingVelocity => super.maxFlingVelocity * 0.4;

//  @override
//  double get minFlingDistance => 10.0;

  @override
  NUICustomScrollPhysics applyTo(ScrollPhysics ancestor) {
    return NUICustomScrollPhysics(parent: buildParent(ancestor));
  }
}

class _NUIListViewState<T, C> extends State<NUIListView> {
  ScrollController _scrollController;
  RefreshController _refreshController;
  double _offset = 0.0;
  NUIListViewModel<T, C> curModel;
  NUIListViewBloc<T, C> bloc;
  List items = List();
  NUIListViewParentHandler actionHandler;
  Map<String, dynamic> checkedMap;
  Map<String, dynamic> configMap;

  final PaginationLoad pagination = PaginationLoad();
  final SwipeRefresh swipeRefresh = SwipeRefresh();

  bool _hasSkeletalLoad(){
    return widget.skeletalInflater != null && widget.skeletalCount > 0;
  }

  @override
  void initState(){
    super.initState();
    checkedMap = widget.checkedMap ?? Map();
    configMap = widget.configMap ?? Map();
    bloc = widget.bloc;
    curModel = NUIListViewModel<T, C>(data: widget.initialList ?? List());
    if(!isNullOrEmpty(widget.initialList)){
      items.addAll(widget.initialList);
    }
    _scrollController = new ScrollController(
        initialScrollOffset: _offset
    );
    _refreshController = new RefreshController(initialRefresh: false);
    _scrollController.addListener(() {
      if (_scrollController.position.maxScrollExtent == _scrollController.position.pixels) {
        log("NUIListViewBuilder -> running pagination loading ... ");
        _onPaginationLoad(cursor: curModel.cursor);
      }
    });

    actionHandler = NUIListViewParentHandler(widget.clickListener, checkedMap, configMap, (){

    }, (){

    });
  }

  dynamic _getItem(int position){
    return items[position];
  }

  Widget _renderListItem(dynamic data, int position){
    EdgeInsets padding;
    if(widget.orientation == Axis.vertical){
      padding = EdgeInsets.only(
        top: position == 0 ? widget.startDivider ?? 0 : 0,
        bottom: position < items.length - 1 ? widget.middleDivider ?? 0 : widget.endDivider ?? 0
      );
    }
    else{
      padding = EdgeInsets.only(
          left: position == 0 ? widget.startDivider ?? 0 : 0,
          right: position < items.length - 1 ? widget.middleDivider ?? 0 : widget.endDivider ?? 0
      );
    }

    if(data is PaginationLoad){
      //Pagination loading
      return Padding(
        padding: padding,
        child: widget.paginationInflater != null ? widget.paginationInflater(position) : NUIListViewProgressWidget(),
      );
    }
    else if(data == null){
      return Padding(
        padding: padding,
        child: widget.skeletalInflater != null ? widget.skeletalInflater(position) : NUIListViewProgressWidget(),
      );
    }

    NUIListItem item = widget.repo.getItemByClass(data.runtimeType);
    if(item == null){
      throw Exception("NUIListViewBuilder -> List item for type : ${data.runtimeType.toString()} is not registered in the repository");
    }

    NUIListActionController handler = NUIListActionController(actionHandler, NUIListItemModel(position, data, checkedMap: checkedMap, configMap: configMap));
    return Padding(
      padding: padding,
      child: item.inflater(data, position, handler),
    );
  }

  bool _isScrolledToBottom(ScrollNotification info){
    return info.metrics.pixels == info.metrics.maxScrollExtent;
  }

  bool _isScrolledToTop(ScrollNotification info){
    return info.metrics.pixels == 0;
  }

  Widget _getListViewBuilder(){
    final builder = ListView.builder(
        physics: widget.scrollPhysics ?? NUICustomScrollPhysics(),
        padding: widget.padding,
        scrollDirection: widget.orientation,
        shrinkWrap: widget.shrinkWrap ?? false,
        controller: _scrollController,
        itemCount: widget.withItemCount ? items.length : null,
        cacheExtent: widget.cacheOffset,
        itemBuilder: (context, position){
          if(position < items.length) {
            return _renderListItem(_getItem(position), position);
          }
          return null;
        });

    if(widget.enableSwipeRefresh){
      return RefreshConfiguration(
        headerTriggerDistance: 100,
        child: SmartRefresher(
          enablePullDown: !curModel.isLoading(),
          header: NUISwipeRefreshHeader(
            height: 100,
          ),
          controller: _refreshController,
          onRefresh: _onSwipeRefresh,
          child: builder,
        ),
      );
    }
    else{
      return builder;
    }
  }

  void _onPaginationLoad({bool isFirst, C cursor}){
    if (isFirst == true || (widget.enablePagination && !curModel.isLoading())) {
      if(isFirst != true && curModel.data != null && curModel.data.length < widget.minimumPaginateCount) {
        log("NUIListViewBuilder -> Minimum pagination threshold of ${widget.minimumPaginateCount} has not been reached, current count at ${curModel.data.length}");
        return;
      }
      log("NUIListViewBuilder -> Initiating pagination loading with cursor : ${cursor != null}");
      curModel.paginationLoading = true;
      bloc.startPaginationLoading(isFirst, curModel.data, cursor);
    }
  }

  void _onSwipeRefresh(){
    if (!curModel.isLoading()) {
      log("NUIListViewBuilder -> Initiating swipe refresh");
      curModel.swipeRefreshing = true;
      bloc.startSwipeRefreshLoading(curModel.data);
    }
  }

  void _onSkeletalLoad(){
    if (!curModel.isLoading()) {
      curModel.skeletalLoading = true;
      bloc.startSkeletalLoading();
    }
  }

  Widget _buildList(){
    return NotificationListener(
      child: _getListViewBuilder(),
      onNotification: (notification){
        if(notification is ScrollNotification){
          _offset = notification.metrics.pixels;
        }
        return true;
      },
    );
  }

  Widget _buildStreamBuilder(){
    return StreamBuilder(
      initialData: NUIListViewModel<T, C>(paginationLoading: true, data: curModel.data, message: null),
      stream: bloc.stream,
      builder: (BuildContext context, AsyncSnapshot snapshot) {
        NUIListViewModel<T, C> model = snapshot.data;

        //Update checked list and config list if specified
        if(model.updateCheckedMap != null) model.updateCheckedMap(checkedMap);
        log("Checked keys : ${checkedMap.keys}");
        if(model.updateConfigMap != null) model.updateConfigMap(configMap);

        log("NUIListView -> Setting new list model for list ${T.toString()} with cursor type ${C.toString()}");
        curModel = model;
        _refreshController.refreshCompleted();
        if (model.paginationLoading) {
          // Handle progress state
          items.remove(pagination);
          items.add(pagination);
          return _buildList();
        }
        else if(model.skeletalLoading){
          items.clear();
          //Add number of skeletal (null) models into the list according to the skeletal count specified
          for(int i = 0; i < widget.skeletalCount; i++){
            items.add(null);
          }
          return _buildList();
        }
        else if(model.success == false && isNullOrEmpty(model.data) && widget.errorHolder != null){
          return widget.errorHolder(model);
        }
        else if(isNullOrEmpty(model.data) && widget.placeHolder != null){
          return widget.placeHolder();
        }
        else {
          // Handle data state
          items.clear();
          if(widget.fixedFirstItem != null){
            items.add(widget.fixedFirstItem());
          }
          items.addAll(model.data);
          if(widget.fixedLastItem != null){
            items.add(widget.fixedLastItem());
          }
          return _buildList();
        }
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    if(_hasSkeletalLoad()){
      _onSkeletalLoad();
    }
    else{
      _onPaginationLoad(isFirst: true, cursor: curModel.cursor);
    }
    return NUIBlocWidget(
        bloc: bloc,
        child: _buildStreamBuilder()
    );
  }
}



