import 'dart:developer';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';

typedef BottomSheetDismissCallback = void Function(Function onDismiss);
typedef BottomSheetBuilder = Widget Function(BuildContext context);

NUIBottomSheet showNUIBottomSheet({@required BuildContext context, bool dismissible:true, Color barrierColor, double opacity: 0.4, @required BottomSheetBuilder content, double elevation, Function onDismiss}){
  return NUIBottomSheet.showNUIBottomSheet(context: context, content: content, dismissible: dismissible, barrierColor: barrierColor, opacity: opacity, elevation: elevation, onDismiss: onDismiss);
}

Future<T> showNUIBottomSheetForResult<T>({@required BuildContext context, bool dismissible:true, Color barrierColor, double opacity: 0.4, @required BottomSheetBuilder content, double elevation, Function onDismiss}){
  return NUIBottomSheet.showNUIBottomSheetForResult<T>(context: context, content: content, dismissible: dismissible, barrierColor: barrierColor, opacity: opacity, elevation: elevation, onDismiss: onDismiss);
}

NUIBottomSheet showNUIBottomSheetByItem({@required BuildContext context, @required NUIBottomSheetItem item}){
  return NUIBottomSheet.showNUIBottomSheetByItem(context: context, item: item);
}

class NUIBottomSheetItem{
  NUIBottomSheetItem({
    String id,
    this.dismissible:true,
    this.barrierColor,
    this.opacity: 0.4,
    @required this.content,
    this.elevation,
    this.onDismiss}){
    this.id = id ?? randomUUID();
  }
  String id;
  bool dismissible;
  Color barrierColor;
  double opacity;
  BottomSheetBuilder content;
  double elevation;
  Function onDismiss;
}

class NUIBottomSheet{
  NUIBottomSheet._builder({@required this.content, @required this.dismiss});

  final BottomSheetBuilder content;
  BottomSheetDismissCallback dismiss;

  static NUIBottomSheet showNUIBottomSheetByItem({@required BuildContext context, @required NUIBottomSheetItem item}){
    return showNUIBottomSheet(context: context, dismissible: item.dismissible, barrierColor: item.barrierColor, opacity: item.opacity, content: item.content, elevation: item.elevation, onDismiss: item.onDismiss);
  }

  static Future<T> showNUIBottomSheetForResult<T>({@required BuildContext context, bool dismissible:true, Color barrierColor, double opacity: 0.4, bool isScrollControlled: false, @required BottomSheetBuilder content, double elevation, Function onDismiss}) async{
    NUIBottomSheet dialog = NUIBottomSheet._builder(content: content, dismiss: (Function innerDismiss){
      log("NUIBottomSheet -> Dialog dismiss not initiated yet");
    });
    try {
      return await showModalBottomSheet<T>(
        barrierColor: barrierColor ?? Colors.black.withOpacity(opacity ?? 0.4),
        isDismissible: dismissible ?? true,
        backgroundColor: NUIColors.NUITransparent,
        context: context,
        elevation: elevation ?? 0,
        isScrollControlled: isScrollControlled ?? false,
        builder: (context) {
          dialog.dismiss = (Function innerDismiss){
            log("NUIBottomSheet -> Dismissing dialog ... ");
            if(onDismiss != null) {
              onDismiss();
            }
            innerDismiss();
            //Navigator.pop(context);
          };
          return Container(
            height: double.infinity,
            alignment: Alignment.bottomCenter,
            child: content(context),
          );
        },
      );
    }catch(e){}

    return null;
  }

  static NUIBottomSheet showNUIBottomSheet({@required BuildContext context, bool dismissible:true, Color barrierColor, double opacity: 0.4, bool isScrollControlled: false, @required BottomSheetBuilder content, double elevation, Function onDismiss}){
    NUIBottomSheet dialog = NUIBottomSheet._builder(content: content, dismiss: (Function innerDismiss){
      log("NUIBottomSheet -> Dialog dismiss not initiated yet");
    });
    showModalBottomSheet(
        barrierColor: barrierColor ?? Colors.black.withOpacity(opacity ?? 0.4),
        isDismissible: dismissible ?? true,
        backgroundColor: NUIColors.NUITransparent,
        context: context,
        elevation: elevation ?? 0,
        isScrollControlled: isScrollControlled ?? false,
        builder: (context) {
          dialog.dismiss = (Function innerDismiss){
            log("NUIBottomSheet -> Dismissing dialog ... ");
            if(onDismiss != null) {
              onDismiss();
            }
            innerDismiss();
            //Navigator.pop(context);
          };
          return Container(
            height: double.infinity,
            alignment: Alignment.bottomCenter,
            child: content(context),
          );
        },
    );

    return dialog;
  }
}

class BottomSheetBarIndicatorUI extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Align(
      child: Container(
        width: 60,
        height: 4,
        decoration: NUIBoxDecoration(color: NUIColors.NUIGrayOutline, borderRadiusCircular: 2, shadowRadius: 0),
      ),
      alignment: Alignment.center,
    );
  }
}

