import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/data/nui_data_util.dart';

typedef ExpandableHeaderInflater<T> = Widget Function(BuildContext context, T data, bool isExpanded);
typedef ExpandableBodyInflater<T> = Widget Function(BuildContext context, T data, bool isExpanded);
typedef ExpandableItemBuilder<T> = NUIExpandableItem<T> Function(T data, int position, NUIExpandableHandler handler);
typedef ExpandableListener<T> = void Function(T data, int position, bool isExpanded);

class NUIExpandableHandler{
  NUIExpandableHandler._builder();
  ExpandableListener _handler;
  bool _isExpanded = false;
  void updateState(bool isExpanded){
    _isExpanded = isExpanded;
    if(_handler != null) _handler(null, null, isExpanded);
  }
  void toggle(){
    this._isExpanded = !this._isExpanded;
    updateState(this._isExpanded);
  }
}

class NUIExpandableItem<T>{
  NUIExpandableItem({@required this.data, @required this.headerInflater, @required this.bodyInflater});
  final T data;
  final ExpandableHeaderInflater<T> headerInflater;
  final ExpandableBodyInflater<T> bodyInflater;
  NUIExpandableHandler _handler;

  void updateState(bool isExpanded){
    _handler?.updateState(isExpanded);
  }
  void toggle(){
    _handler?.toggle();
  }
}

class NUIExpandable<T> extends StatefulWidget {
  NUIExpandable({this.children, this.items, this.builder, this.listener});
  final List<NUIExpandableItem<T>> children;
  final List<T> items;
  final ExpandableItemBuilder<T> builder;
  final ExpandableListener<T> listener;

  @override
  _NUIExpandableState<T> createState() => _NUIExpandableState<T>();
}

class _NUIExpandableState<T> extends State<NUIExpandable<T>> {
  List<T> items;

  @override
  void initState(){
    super.initState();
    this.items = widget.items ?? List();
  }

  List<ExpansionPanel> _buildChildren(){
    if(!isNullOrEmpty(widget.children)){
      //Load children
      return widget.children.map((child){
        final NUIExpandableHandler handler = child._handler ?? NUIExpandableHandler._builder();
        handler._handler = (data, position, isExpanded){
          setState(() {

          });
        };

        child._handler = handler;
        return ExpansionPanel(
          headerBuilder: (BuildContext context, bool isExpanded) {
            return child.headerInflater(context, child.data, isExpanded);
          },
          isExpanded: child._handler._isExpanded,
          body: child.bodyInflater(context, child.data, child._handler._isExpanded)
        );
      });
    }
    else if(!isNullOrEmpty(items) && widget.builder != null){
      //Build children one by one
      int position = 0;
      items.map((item){
        final NUIExpandableHandler handler = NUIExpandableHandler._builder();
        handler._handler = (data, position, isExpanded){
          setState(() {

          });
        };

        final expandItem = widget.builder(item, position, handler);
        position ++;

        expandItem._handler = handler;
        return ExpansionPanel(
            headerBuilder: (BuildContext context, bool isExpanded) {
              return expandItem.headerInflater(context, expandItem.data, isExpanded);
            },
            isExpanded: expandItem._handler._isExpanded,
            body: expandItem.bodyInflater(context, expandItem.data, expandItem._handler._isExpanded)
        );
      });
    }
    else{
      return List();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ExpansionPanelList(
      expansionCallback: (int index, bool isExpanded) {
        if(widget.listener != null){
          final T item = items[index];
          widget.listener(item, index, isExpanded);
        }
        setState(() {});
      },
      children: _buildChildren()
    );
  }
}
