import 'package:flutter/cupertino.dart';
import 'package:nui_core/model/gradient_background.dart';
import 'package:nui_core/nui_core.dart';

typedef GradientBackgroundSelectListener = void Function(GradientBackground background);

class NUIGradientColorSelection{
  NUIGradientColorSelection({@required this.background});
  GradientBackground background;
}

class NUIGradientColorSelectionSkeletal extends StatelessWidget {
  NUIGradientColorSelectionSkeletal({this.size : 20, this.selectedSize: 23});
  final double size;
  final double selectedSize;
  @override
  Widget build(BuildContext context) {
    final finalSize = size ?? 20;
    final finalSelectedSize = selectedSize ?? 23;
    return Container(
      width: finalSize,
      height: finalSize,
      decoration: NUIBoxDecoration(
          gradient: NUILinearGradient(
              colors: [NUIColors.NUISkeletalGray, NUIColors.NUISkeletalLightGray],
              orientation: GradientOrientation.T_B
          ),
          borderRadiusCircular: finalSize/2,
          shape: BoxShape.circle
      ),
    );
  }
}

class NUIGradientColorSelectionItem extends NUIListStatefulWidget {
  NUIGradientColorSelectionItem({this.color, this.size, this.selectedSize, NUIListActionController handler}): super(controller: handler);
  final NUIGradientColorSelection color;
  final double size;
  final double selectedSize;
  @override
  _NUIGradientColorSelectionItemUIState createState() => _NUIGradientColorSelectionItemUIState();
}

class _NUIGradientColorSelectionItemUIState extends NUIListState<NUIGradientColorSelectionItem> {
  @override
  Widget build(BuildContext context) {
    final size = getConfigData("size") ?? widget.size ?? 20.0;
    final finalSelectedSize = getConfigData("selectedSize") ?? widget.selectedSize ?? 23;
    return GestureDetector(
      child: Container(
        child: Container(
          width: size,
          height: size,
          decoration: NUIBoxDecoration(
            gradient: NUILinearGradient(
              colors: widget.color.background.colors(),
              orientation: GradientOrientation.T_B
            ),
            borderRadiusCircular: size/2,
            shape: BoxShape.circle
          ),
        ),
      ),
      onTap: (){
        getController().onItemClick(widget.color, NUIListActions.NORMAL);
      },
    );
  }
}

class NUIGradientColorSelectionView extends StatefulWidget {
  NUIGradientColorSelectionView({this.colors, this.size: 20, this.listener, this.padding, this.divider: 15});
  final Future<List<GradientBackground>> colors;
  final double size;
  final EdgeInsets padding;
  final double divider;
  final GradientBackgroundSelectListener listener;
  @override
  _NUIGradientColorSelectionViewState createState() => _NUIGradientColorSelectionViewState();
}

class _NUIGradientColorSelectionViewState extends State<NUIGradientColorSelectionView> {
  NUIListViewBloc<NUIGradientColorSelection, NUIGradientColorSelection> bloc;
  @override
  void initState(){
    super.initState();
    bloc = NUIListViewBloc<NUIGradientColorSelection, NUIGradientColorSelection>(callbackMethod: (List<NUIGradientColorSelection> current, bool isFirst, bool append, NUIGradientColorSelection cursor) async{
      final colors = widget.colors != null ? await widget.colors : NUIGradientBackgrounds.getDefaultGradientBackgrounds();
      final result = ListResult<NUIGradientColorSelection, NUIGradientColorSelection>(data: colors.map((e) => NUIGradientColorSelection(background: e)).toList(), success: true, message: "Get color selections successful", cursor: null);
      return result;
    });
  }

  @override
  Widget build(BuildContext context) {
    final size = widget.size ?? 20;
    final padding = widget.padding ?? const EdgeInsets.only(left: 5, top: 5, bottom: 5, right: 5);
    return Container(
      height: size + padding.top + padding.bottom,
      child: NUIListView(
        bloc: bloc,
        repo: NUIInternalListItemRepo(),
        enablePagination: false,
        enableSwipeRefresh: false,
        skeletalCount: 2,
        shrinkWrap: true,
        configMap: {
          "size": size
        },
        orientation: Axis.horizontal,
        padding: padding,
        middleDivider: widget.divider ?? 15,
        skeletalInflater: (int position) => NUIGradientColorSelectionSkeletal(),
        initialList: List<NUIGradientColorSelection>(),
        clickListener: (data, position, action){
          if(match(action, NUIListActions.NORMAL)){
            final color = data as NUIGradientColorSelection;
            if(widget.listener != null) widget.listener(color.background);
          }
        },
      ),
    );
  }
}
