import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:nui_core/nui_core.dart';

typedef NUINumberPadOnDone = void Function(String value, bool complete);

class NUINumberPad extends StatefulWidget {
  NUINumberPad({this.numOfDigits : 4, this.highlightColor : NUIColors.NUIAccent, this.inputTextStyle, this.valueTextStyle, this.inputActionStyle, this.defaultColor : NUIColors.NUITextGray, this.showEnteredNumbers: true, this.onDone});
  final int numOfDigits;
  final Color highlightColor;
  final Color defaultColor;
  final bool showEnteredNumbers;
  final TextStyle valueTextStyle;
  final TextStyle inputTextStyle;
  final TextStyle inputActionStyle;
  final NUINumberPadOnDone onDone;
  @override
  _NUINumberPadState createState() => _NUINumberPadState();
}

class _NUINumberPadState extends State<NUINumberPad> {
  final inputs = ["1", "2", "3", "4", "5", "6", "7", "8", "9", "C", "0", "OK"];
  String numbersEntered = "";

  void _onButtonClick(String value){
    if(match(value, "C")){
      //Cancel
      if(numbersEntered.length > 0){
        numbersEntered = numbersEntered.substring(0, numbersEntered.length - 1);
      }
      else{
        numbersEntered = "";
      }
    }
    else if(match(value, "OK")){
      //OK
      if(widget.onDone != null) widget.onDone(numbersEntered, numbersEntered.length == widget.numOfDigits);
    }
    else{
      //Normal numbers
      if(numbersEntered.length < widget.numOfDigits){
        numbersEntered = numbersEntered + value;
      }
    }
    setState(() {

    });
  }

  Widget _generateEnteredNumbers(){
    if(widget.showEnteredNumbers == false) return null;

    final inputs = List<Widget>();
    for(int i = 0; i< widget.numOfDigits; i++){
      final char = numbersEntered.length > i ? numbersEntered[i] : null;
      inputs.add(_NUINumberPadValueText(number: char != null ? int.parse(char) : null, highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, valueTextStyle: widget.valueTextStyle,));
    }
    return Padding(
      padding: const EdgeInsets.only(left: 10, right: 10),
      child: NUIRow(
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: inputs,
          divider: VerticalDivider(width: (4.toDouble()/widget.numOfDigits)* 20, color: NUIColors.NUITransparent)
      ),
    );
  }

  Widget _generateButtons(){
    return NUIColumn(
        mainAxisSize: MainAxisSize.min,
        children: [
          _generateEnteredNumbers(),
          NUIRow(
              mainAxisSize: MainAxisSize.max,
              children: [
                NUINumberPadInput(input: "1", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "2", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "3", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
              ],
              divider: VerticalDivider(width: 20, color: NUIColors.NUITransparent)
          ),
          NUIRow(
              mainAxisSize: MainAxisSize.max,
              children: [
                NUINumberPadInput(input: "4", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "5", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "6", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
              ],
              divider: VerticalDivider(width: 20, color: NUIColors.NUITransparent)
          ),
          NUIRow(
              mainAxisSize: MainAxisSize.max,
              children: [
                NUINumberPadInput(input: "7", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "8", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "9", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
              ],
              divider: VerticalDivider(width: 20, color: NUIColors.NUITransparent)
          ),
          NUIRow(
              mainAxisSize: MainAxisSize.max,
              children: [
                NUINumberPadInput(input: "C", highlightColor: widget.highlightColor, outline: true, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "0", highlightColor: widget.highlightColor, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
                NUINumberPadInput(input: "OK", highlightColor: widget.highlightColor, outline: true, defaultColor: widget.defaultColor, inputActionStyle: widget.inputActionStyle, inputTextStyle: widget.inputTextStyle, onClick: _onButtonClick),
              ],
              divider: VerticalDivider(width: 20, color: NUIColors.NUITransparent)
          ),
        ],
        divider: Divider(height: 20, color: NUIColors.NUITransparent)
    );
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      child: _generateButtons(),
    );
  }
}

class _NUINumberPadValueText extends StatefulWidget {
  _NUINumberPadValueText({this.number, this.highlightColor : NUIColors.NUIAccent, this.defaultColor: NUIColors.NUITextGray, this.valueTextStyle});
  final int number;
  final Color highlightColor;
  final TextStyle valueTextStyle;
  final Color defaultColor;
  @override
  __NUINumberPadValueTextState createState() => __NUINumberPadValueTextState();
}

class __NUINumberPadValueTextState extends State<_NUINumberPadValueText> {
  @override
  Widget build(BuildContext context) {
    final hasValue = widget.number != null;
    return Expanded(
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Container(
            padding: EdgeInsets.only(left: 15, right: 15, top: 10, bottom: 10),
            alignment: Alignment.center,
            child: Text(widget.number?.toString() ?? "", style: widget.valueTextStyle ?? getFont(context, singleColor: widget.highlightColor, size: 24, isBold: true)),
          ),
          Divider(height: 3, color: hasValue ? widget.highlightColor : widget.defaultColor, thickness: 3)
        ],
      ),
    );
  }
}

typedef NUINumberPadOnClick = void Function(String value);
class NUINumberPadInput extends StatefulWidget {
  NUINumberPadInput({this.input, this.onClick, this.outline: false, this.inputActionStyle, this.inputTextStyle, this.defaultColor: NUIColors.NUITextGray, this.highlightColor: NUIColors.NUIAccent});
  final String input;
  final bool outline;
  final Color highlightColor;
  final Color defaultColor;
  final TextStyle inputActionStyle;
  final TextStyle inputTextStyle;
  final NUINumberPadOnClick onClick;
  @override
  _NUINumberPadInputState createState() => _NUINumberPadInputState();
}

class _NUINumberPadInputState extends State<NUINumberPadInput> {
  @override
  Widget build(BuildContext context) {
    return Expanded(
      child: AspectRatio(
        aspectRatio: 1,
        child: Container(
          decoration: NUIOutlineDecoration(
              thickness: widget.outline == true ? 2 : 0,
              outlineColor: widget.outline == true ? (widget.highlightColor ?? NUIColors.NUIAccent) : NUIColors.NUITransparent,
              borderRadius: BorderRadius.circular(100)
          ),
          child: NUIRippleContainer(
            borderRadius: BorderRadius.circular(100),
            child: Center(
              child: Text(widget.input, style: (widget.outline != true ? widget.inputTextStyle : widget.inputActionStyle) ?? getFont(context, size: 16, singleColor: widget.outline != true ? widget.defaultColor : widget.highlightColor)),
            ),
            onTap: (){
              widget.onClick(widget.input);
            },
          ),
        ),
      ),
    );
  }
}
