import 'package:advance_pdf_viewer/advance_pdf_viewer.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:line_awesome_flutter/line_awesome_flutter.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';

typedef NUIPdfDocumentRetriever = Future<PDFDocument> Function();

class NUIPdfViewer extends StatefulWidget {
  NUIPdfViewer({@required this.documentRetriever});
  final NUIPdfDocumentRetriever documentRetriever;
  @override
  _NUIPdfViewerState createState() => _NUIPdfViewerState();
}

class _NUIPdfViewerState extends State<NUIPdfViewer> {

  Future<PDFDocument> document;

  @override
  void initState(){
    super.initState();
    this.document = widget.documentRetriever();
  }

  void _retry(){
    this.document = widget.documentRetriever();
    setState(() {

    });
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: double.infinity,
      height: double.infinity,
      color: NUIColors.NUIWhite,
      child: NUIFutureWidget<PDFDocument>(
        futureValue: document,
        callback: (state, data, message){
          if(state == FutureState.SUCCESS && data != null){
            //File obtained
            return PDFViewer(document: data, showPicker: false,);
          }
          else if(state == FutureState.LOADING){
            //Loading pdf
            return SizedBox(
              height: 1.5,
              width: double.infinity,
              child: LinearProgressIndicator(backgroundColor: NUIColors.NUISkeletalGray, valueColor: AlwaysStoppedAnimation<Color>(NUIColors.NUITextBlack)),
            );
          }
          else{
            //Failed to load pdf
            return NUIColumn(
              crossAxisAlignment: CrossAxisAlignment.center,
              mainAxisAlignment: MainAxisAlignment.center,
              mainAxisSize: MainAxisSize.max,
              children: [
                NUIRippleContainer(
                  borderRadius: BorderRadius.circular(100),
                  rippleColor: NUIColors.NUIBlack.withOpacity(0.1),
                  child: Padding(
                    padding: const EdgeInsets.all(10.0),
                    child: Icon(
                      LineAwesomeIcons.redo,
                      color: NUIColors.NUIDarkGrayOutline,
                      size: 50,
                    ),
                  )
                ),
                Text(
                  "Retry",
                  style: getFont(context, size: 16, singleColor: NUIColors.NUIDarkGrayOutline, isBold: true),
                )
              ],
              divider: Divider(height: 10, color: NUIColors.NUITransparent)
            );
          }
        },
      ),
    );
  }
}
