import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/nui_core.dart';

typedef NUISearchTrigger = void Function(String keyword);

class NUISearchTextController{
  NUISearchTextController({@required this.searchTrigger});
  Function _startProcessingTask;
  Function _endProcessingTask;
  NUISearchTrigger searchTrigger;
  String _keyword;

  String getSearchKeyword(){
    return _keyword;
  }

  void startSearching(){
    if(_startProcessingTask != null) _startProcessingTask();
  }

  void endSearching(){
    if(_endProcessingTask != null) _endProcessingTask();
  }

  void _initSearching(String keyword){
    if(searchTrigger != null) searchTrigger(keyword);
  }
}

class NUISearchText extends StatefulWidget {
  NUISearchText({
    this.autoFocus: false,
    this.autoCorrect: false,
    this.textInputType: TextInputType.name,
    this.enabled: true,
    this.textStyle,
    this.textCapitalization: TextCapitalization.sentences,
    this.textAlign: TextAlign.left,
    this.cursorColor: NUIColors.NUITextBlack,
    this.cursorRadius,
    this.cursorWidth: 2,
    this.maxLength,
    this.inputDecoration,
    this.hint,
    this.initialValue : "",
    this.controller,
    this.hintTextStyle,
    this.searchOnChanged: false,
    this.searchDelay
  });
  final bool autoFocus;
  final bool autoCorrect;
  final TextInputType textInputType;
  final bool enabled;
  final TextCapitalization textCapitalization;
  final TextAlign textAlign;
  final TextStyle textStyle;
  final TextStyle hintTextStyle;
  final Color cursorColor;
  final double cursorWidth;
  final Radius cursorRadius;
  final int maxLength;
  final InputDecoration inputDecoration;
  final String hint;
  final String initialValue;
  final NUISearchTextController controller;
  final Duration searchDelay;
  final bool searchOnChanged;
  @override
  _NUISearchTextState createState() => _NUISearchTextState();
}

class _NUISearchTextState extends State<NUISearchText> {
  TextEditingController controller;
  NUISearchTextController searchTextController;
  bool _processing = false;

  void _onChanged(String value){
    searchTextController._keyword = value;
    if(_processing == true || widget.searchOnChanged != true) return;

    final delayValue = value;
    NUIAsync.delay(widget.searchDelay ?? Duration(seconds: 2), () {
        if(match(controller.text, delayValue) && _processing != true){
          _initSearch(delayValue);
        }
    });
  }

  void _initSearch(String keyword){
    logNUI("NUISearchText", "Initiating search with keyword : $keyword");
    _processing = true;
    searchTextController._initSearching(keyword);
  }

  void _onEditingComplete(){

  }

  void _onSubmitted(String value){

  }

  @override
  void initState(){
    super.initState();
    searchTextController = widget.controller ?? NUISearchTextController();
    controller = TextEditingController(text: widget.initialValue);

    searchTextController._startProcessingTask = (){
      if(_processing != true) {
        _processing = true;
      }
    };
    searchTextController._endProcessingTask = (){
      if(_processing != false) {
        _processing = false;
      }
    };
  }

  @override
  Widget build(BuildContext context) {
    return TextField(
      controller: controller,
      onChanged: _onChanged,
      onEditingComplete: _onEditingComplete,
      onSubmitted: _onSubmitted,
      autofocus: widget.autoFocus,
      autocorrect: widget.autoCorrect,
      keyboardType: widget.textInputType ?? TextInputType.text,
      textInputAction: TextInputAction.search,
      textCapitalization: widget.textCapitalization ?? TextCapitalization.sentences,
      enabled: widget.enabled,
      textAlign: widget.textAlign,
      style: widget.textStyle,
      cursorColor: widget.cursorColor,
      cursorRadius: widget.cursorRadius,
      cursorWidth: widget.cursorWidth,
      maxLength: widget.maxLength,
      maxLines: 1,
      decoration: widget.inputDecoration ?? InputDecoration.collapsed(
        hintText: widget.hint ?? "",
        hintStyle: widget.hintTextStyle ?? getFont(context, singleColor: NUIColors.NUITextGray),
      ),
    );
  }
}
