import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

enum NUIShaderMaskType{
  TOP_FADE,
  BOTTOM_FADE,
  TOP_BOTTOM_FADE,
  LEFT_FADE,
  RIGHT_FADE,
  LEFT_RIGHT_FADE,
  NONE
}

class NUIShaderMask extends StatelessWidget {
  NUIShaderMask({@required this.child, @required this.shaderType, this.shadeOffset: 0.1});
  final Widget child;
  final NUIShaderMaskType shaderType;
  final double shadeOffset;

  LinearGradient _getShaderGradient(){
    final toUseShadeOffset = shadeOffset ?? 0.1;
    final shadeStops = [0.0, toUseShadeOffset, 1.0 - shadeOffset, 1.0];
    if(shaderType == NUIShaderMaskType.NONE){
      return LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [Colors.transparent, Colors.transparent, Colors.transparent, Colors.transparent],
        stops: shadeStops, // 10% purple, 80% transparent, 10% purple
      );
    }
    else if(shaderType == NUIShaderMaskType.TOP_FADE || shaderType == NUIShaderMaskType.BOTTOM_FADE || shaderType == NUIShaderMaskType.TOP_BOTTOM_FADE){
      final fadeTop = shaderType == NUIShaderMaskType.TOP_FADE || shaderType == NUIShaderMaskType.TOP_BOTTOM_FADE;
      final fadeBottom = shaderType == NUIShaderMaskType.BOTTOM_FADE || shaderType == NUIShaderMaskType.TOP_BOTTOM_FADE;
      return LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [fadeTop ? Colors.black : Colors.transparent, Colors.transparent, Colors.transparent, fadeBottom ? Colors.black : Colors.transparent],
        stops: shadeStops, // 10% purple, 80% transparent, 10% purple
      );
    }
    else{
      final fadeLeft = shaderType == NUIShaderMaskType.LEFT_FADE || shaderType == NUIShaderMaskType.LEFT_RIGHT_FADE;
      final fadeRight = shaderType == NUIShaderMaskType.RIGHT_FADE || shaderType == NUIShaderMaskType.LEFT_RIGHT_FADE;
      return LinearGradient(
        begin: Alignment.centerLeft,
        end: Alignment.centerRight,
        colors: [fadeLeft ? Colors.black : Colors.transparent, Colors.transparent, Colors.transparent, fadeRight ? Colors.black : Colors.transparent],
        stops: shadeStops, // 10% purple, 80% transparent, 10% purple
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    return ShaderMask(
      shaderCallback: (Rect rect) {
        return _getShaderGradient().createShader(rect);
      },
      blendMode: BlendMode.dstOut,
      child: child
    );
  }
}
