import 'package:flutter/cupertino.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:nui_core/model/enums/gradient_orientation.dart';
import 'package:nui_core/ui/styling/nui_box_decoration.dart';
import 'package:nui_core/ui/styling/nui_linear_gradient.dart';

class NUIShadowLinearView extends StatelessWidget {
  NUIShadowLinearView({this.shadowColor: NUIColors.NUIBlack, this.opacity: 0.7, @required this.height, @required this.width, this.orientation});
  final Color shadowColor;
  final double opacity;
  final double height;
  final double width;
  final GradientOrientation orientation;
  @override
  Widget build(BuildContext context) {
    return Container(
      width: width,
      height: height,
      decoration: NUIBoxDecoration(
          gradient: NUILinearGradient(
              colors: [(shadowColor ?? NUIColors.NUIBlack).withOpacity(0.7), NUIColors.NUITransparent],
              orientation: orientation ?? GradientOrientation.T_B
          )
      ),
    );
  }
}


class NUIShadowView extends StatelessWidget {
  NUIShadowView({this.color, this.opacity, this.blurRadius, this.spreadRadius, this.offset, this.radius:0, @required this.child, this.padding, this.margin});
  final Color color;
  final double opacity;
  final double blurRadius;
  final double spreadRadius;
  final Offset offset;
  final Widget child;
  final EdgeInsets padding;
  final EdgeInsets margin;
  final double radius;

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: padding,
      margin: margin,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(radius ?? 0),
        boxShadow: [
          BoxShadow(
            color: (color ?? NUIColors.NUIBlack).withOpacity(opacity ?? 0.2),
            spreadRadius: spreadRadius ?? 0,
            blurRadius: blurRadius ?? 5,
            offset: offset ?? Offset(0, 0), // changes position of shadow
          ),
        ],
      ),
      child: child,
    );
  }
}
