import 'package:flutter/cupertino.dart';
import 'package:nui_core/color/nui_colors.dart';
import 'package:flutter/rendering.dart';

class NUISkeletonView extends StatefulWidget {
  NUISkeletonView({Key key, this.height, this.width, this.child, this.margin, this.radius, this.padding, this.color:NUIColors.NUISkeletalLightGray}) : super(key: key);
  final double height;
  final double width;
  final Widget child;
  final EdgeInsets margin;
  final EdgeInsets padding;
  final Color color;
  final BorderRadius radius;
  createState() => NUISkeletonViewState();
}

class NUISkeletonViewState extends State<NUISkeletonView> with SingleTickerProviderStateMixin {

  Widget _getChildIfValid(){
    if(widget.child == null && widget.width != null && widget.height != null){
      return SizedBox(width: widget.width, height: widget.height,);
    }
    return widget.child;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: widget.margin,
      padding: widget.padding,
      decoration: BoxDecoration(
          borderRadius: widget.radius ?? BorderRadius.circular(5),
          color: widget.color),
      child: _getChildIfValid(),
    );
  }
}


//shimmer class
class NUISkeletonContainer extends StatefulWidget {
  NUISkeletonContainer({
    @required this.child,
    Color baseColor,
    Color highlightColor,
    this.duration = const Duration(milliseconds: 1500),
  }) : gradient = LinearGradient(
    begin: Alignment.topLeft,
    end: Alignment.centerRight,
    colors: [baseColor ?? NUIColors.NUISkeletalLightGray, baseColor ?? NUIColors.NUISkeletalLightGray, highlightColor ?? NUIColors.NUISkeletalGray, baseColor ?? NUIColors.NUISkeletalLightGray, baseColor ?? NUIColors.NUISkeletalLightGray],
    stops: const [0.0, 0.2, 0.5, 0.8, 1.0],
  );

  final Widget child;
  final Duration duration;
  final Gradient gradient;

  @override
  _NUISkeletonContainerState createState() => _NUISkeletonContainerState();
}

class _NUISkeletonContainerState extends State<NUISkeletonContainer> with TickerProviderStateMixin {
  AnimationController _controller;

  @override
  void initState() {
    super.initState();
    _controller = AnimationController(vsync: this, duration: widget.duration)
      ..addListener(() {
        setState(() {});
      })
      ..repeat();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => _Shimmer(
    child: widget.child,
    gradient: widget.gradient,
    percent: _controller.value,
  );
}

class _Shimmer extends SingleChildRenderObjectWidget {
  _Shimmer({Widget child, this.gradient, this.percent}) : super(child: child);

  final Gradient gradient;
  final double percent;

  @override
  RenderObject createRenderObject(BuildContext context) => _ShimmerFilter(gradient);

  @override
  void updateRenderObject(BuildContext context, RenderObject renderObject) {
    super.updateRenderObject(context, renderObject);
    (renderObject as _ShimmerFilter).shiftPercentage = percent;
  }
}

class _ShimmerFilter extends RenderProxyBox {
  _ShimmerFilter(this._gradient);

  final Gradient _gradient;
  double _shiftPercentage = 0.0;

  set shiftPercentage(double newValue) {
    if (_shiftPercentage != newValue) {
      _shiftPercentage = newValue;
      markNeedsPaint();
    }
  }

  @override
  ShaderMaskLayer get layer => super.layer;

  @override
  bool get alwaysNeedsCompositing => child != null;

  @override
  void paint(PaintingContext context, Offset offset) {
    if (child != null) {
      assert(needsCompositing);

      final width = child.size.width;
      final height = child.size.height;
      double dx = _offset(start: -width, end: width * 2, percent: _shiftPercentage);
      double dy = 0.0;
      final rect = Rect.fromLTWH(dx, dy, width, height);

      layer ??= ShaderMaskLayer();
      layer
        ..shader = _gradient.createShader(rect)
        ..maskRect = offset & size
        ..blendMode = BlendMode.srcIn;
      context.pushLayer(layer, super.paint, offset);
    }
  }

  double _offset({double start, double end, double percent}) => start + (end - start) * percent;
}