import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nui_core/nui_core.dart';

typedef NUIFolderOnSelectListener = void Function(int position);
typedef NUIFolderTabItemBuilder = Widget Function(int position, String title, bool selected);

class NUIFolderStyleTabBar extends StatefulWidget {
  static Path getFolderStylePath(Size size, {double topLeftRadius : 4, double slopeMargin: 20}){
    final topLeftRad = topLeftRadius ?? 4;
    final slopeMarg = slopeMargin ?? 20;
    final path = Path();
    path.lineTo(topLeftRad, 0);
    path.lineTo(size.width - slopeMarg, 0);
    path.lineTo(size.width, size.height);
    path.lineTo(0, size.height);
    path.lineTo(0, topLeftRad);
    path.arcToPoint(Offset(topLeftRad, 0), radius: Radius.circular(topLeftRad), rotation: 270);
    path.close();
    return path;
  }

  NUIFolderStyleTabBar({this.selectedTitleStyle, this.listener, this.itemBuilder, this.unselectedTitleStyle, this.selectedTabColor, this.tabBarPadding, this.shadowColor, this.shadowRadius, this.tabItemPadding, @required this.tabsTitles, this.initialTab: 0, this.topLeftRadius: 4, this.slopeMargin: 20});
  final List<String> tabsTitles;
  final TextStyle selectedTitleStyle;
  final TextStyle unselectedTitleStyle;
  final double topLeftRadius;
  final double slopeMargin;
  final int initialTab;
  final EdgeInsets tabItemPadding;
  final EdgeInsets tabBarPadding;
  final Color shadowColor;
  final double shadowRadius;
  final Color selectedTabColor;
  final NUIFolderOnSelectListener listener;
  final NUIFolderTabItemBuilder itemBuilder;

  @override
  _NUIFolderStyleTabBarState createState() => _NUIFolderStyleTabBarState();
}

class _NUIFolderStyleTabBarState extends State<NUIFolderStyleTabBar> {
  int _selectedTab;

  @override
  void initState(){
    super.initState();
    _selectedTab = widget.initialTab ?? 0;
  }

  Widget _renderTab(int position, {String tabTitle}){
    final isSelected = _selectedTab == position;

    final customChild = widget.itemBuilder != null ? widget.itemBuilder(position, tabTitle, position == _selectedTab) : null;

    final tabView = GestureDetector(
      child: Container(
        padding: widget.tabItemPadding ?? EdgeInsets.only(top: 12, bottom: 12, left: 25, right: 45),
        color: isSelected ? (widget.selectedTabColor ?? Colors.white) : Colors.transparent,
        alignment: Alignment.center,
        child: customChild ?? Text(tabTitle, style: isSelected ? (widget.selectedTitleStyle ?? getFont(context, size: 16, singleColor: NUIColors.NUITextBlack, isBold: true)) : (widget.unselectedTitleStyle ?? getFont(context, size: 14, singleColor: NUIColors.NUITextGray, isBold: true))),
      ),
      onTap: (){
        setState(() {
          _selectedTab = position;
          if(widget.listener != null) widget.listener(position);
        });
      },
    );

    if(!isSelected) return tabView;

    return CustomPaint(
        painter: _NUIFolderStyleBoxShadowPainter(
          topLeftRadius: widget.topLeftRadius,
          slopeMargin: widget.slopeMargin,
          color: widget.shadowColor ?? Colors.black.withOpacity(0.2),
          shadowRadius: widget.shadowRadius ?? 10,
        ),
        child: ClipPath(
            clipper: _NUIFolderStyleTabClipper(topLeftRadius: widget.topLeftRadius, slopeMargin: widget.slopeMargin), //my CustomClipper
            child: tabView
        )
    );
  }

  @override
  Widget build(BuildContext context) {
    final children = List<Widget>();

    int position = 0;
    for(String tab in widget.tabsTitles){
      final child = _renderTab(position, tabTitle: tab);
      children.add(child);
      position++;
    }

    return Stack(
      children: [
        Align(
          alignment: Alignment.bottomCenter,
          child: Container(
            width: double.infinity,
            height: widget.shadowRadius ?? 10,
            decoration: NUIBoxDecoration(
//              gradient: NUILinearGradient(
//                colors: [
//                  Colors.transparent,
//                  Colors.transparent,
//                  widget.shadowColor?.withOpacity(0.2) ?? Colors.black.withOpacity(0.2),
//                ],
//                orientation: GradientOrientation.T_B
//              )
                shadowOffset: Offset(0, (widget.shadowRadius ?? 10) * 1.3),
                shadowColor: widget.shadowColor ?? Colors.black.withOpacity(0.2),
                shadowRadius: widget.shadowRadius ?? 10
            ),
          ),
        ),
        ListView(
            padding: widget.tabBarPadding ?? EdgeInsets.only(left: 30, right: 30),
            scrollDirection: Axis.horizontal,
            children: children
        ),
      ],
    );
  }
}


class _NUIFolderStyleTabClipper extends CustomClipper<Path> {
  _NUIFolderStyleTabClipper({this.topLeftRadius: 4, this.slopeMargin: 20});
  final double topLeftRadius;
  final double slopeMargin;
  @override
  Path getClip(Size size) {
    return NUIFolderStyleTabBar.getFolderStylePath(size, topLeftRadius: topLeftRadius, slopeMargin: slopeMargin);
  }

  @override
  bool shouldReclip(CustomClipper<Path> oldClipper) => false;
}

class _NUIFolderStyleBoxShadowPainter extends CustomPainter {
  _NUIFolderStyleBoxShadowPainter({this.topLeftRadius: 4, this.slopeMargin: 20, this.color, this.shadowRadius});
  final Color color;
  final double shadowRadius;
  final double topLeftRadius;
  final double slopeMargin;

  @override
  void paint(Canvas canvas, Size size) {
    final path = NUIFolderStyleTabBar.getFolderStylePath(size, topLeftRadius: topLeftRadius, slopeMargin: slopeMargin);
    canvas.drawShadow(path, color ?? Colors.black.withOpacity(0.2), shadowRadius?? 3.0, false);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}