import 'package:flutter/material.dart';
import 'package:nui_core/color/nui_colors.dart';
import '../nui_ripple_container.dart';

class NUIToolbar extends StatefulWidget {
  NUIToolbar({@required this.content, this.backgroundColor: NUIColors.NUIWhite, this.buttonSize:36, this.titleGravity:Alignment.centerLeft, this.titlePadding: 10, this.decoration, this.toolbarHeight: 48, this.leftButton, this.rightButton, Key key}) : super(key: key);
  final double titlePadding;
  final Alignment titleGravity;
  final double buttonSize;
  final double toolbarHeight;
  final Widget content;
  final Widget leftButton;
  final BoxDecoration decoration;
  final Color backgroundColor;
  final List<Widget> rightButton;
  @override
  _NUIToolbarState createState() => _NUIToolbarState();
}

class _NUIToolbarState extends State<NUIToolbar> {
  @override
  Widget build(BuildContext context) {

    final titleView = Expanded(
        child: Container(
          padding: EdgeInsets.only(left: widget.titlePadding, right: widget.titlePadding),
          alignment: widget.titleGravity,
          width: double.infinity,
          child: widget.content,
        )
    );

    final toolbarChildren = List<Widget>();
    if(widget.leftButton != null){
      toolbarChildren.add(widget.leftButton);
    }
    toolbarChildren.add(titleView);
    if(widget.rightButton != null && widget.rightButton.isNotEmpty){
      final buttonRow = Row(
        mainAxisSize: MainAxisSize.min,
        children: widget.rightButton,
      );
      toolbarChildren.add(buttonRow);
    }

    return Container(
      width: double.infinity,
      height: widget.toolbarHeight,
      decoration: widget.decoration,
      child: Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: toolbarChildren
      ),
    );
  }
}


//Toolbar image/icon button
class NUIToolbarIconButton extends StatefulWidget {
  NUIToolbarIconButton({@required this.image, @required this.color, @required this.radius, @required this.onTap, this.padding, this.rippleColor: NUIColors.NUIShadowBlackTransparent, Key key}) : super(key: key);
  final double radius;
  final Widget image;
  final Color color;
  final Color rippleColor;
  final EdgeInsets padding;
  final GestureTapCallback onTap;
  @override
  _NUIToolbarIconButtonState createState() => _NUIToolbarIconButtonState();
}

class _NUIToolbarIconButtonState extends State<NUIToolbarIconButton> {
  @override
  Widget build(BuildContext context) {
    return NUIRippleContainer(
      radius: widget.radius ?? 0,
      borderRadius: BorderRadius.circular(widget.radius ?? 0),
      rippleColor: widget.rippleColor ?? NUIColors.NUIBlack.withOpacity(0.1),
      onTap: widget.onTap,
      child: Container(
        padding: widget.padding ?? EdgeInsets.all(0),
        alignment: Alignment.center,
        child: widget.image,
      ),
    );
  }
}


//Toolbar text button
class NUIToolbarTextButton extends StatefulWidget {
  NUIToolbarTextButton({@required this.text, @required this.color, @required this.radius, @required this.onTap, this.padding, this.rippleColor: NUIColors.NUIShadowBlackTransparent, Key key}) : super(key: key);
  final double radius;
  final Text text;
  final Color color;
  final Color rippleColor;
  final EdgeInsets padding;
  final GestureTapCallback onTap;
  @override
  _NUIToolbarTextButtonState createState() => _NUIToolbarTextButtonState();
}

class _NUIToolbarTextButtonState extends State<NUIToolbarTextButton> {
  @override
  Widget build(BuildContext context) {
    return NUIRippleContainer(
      radius: widget.radius ?? 0,
      borderRadius: BorderRadius.circular(widget.radius ?? 0),
      rippleColor: widget.rippleColor ?? NUIColors.NUIBlack.withOpacity(0.1),
      onTap: widget.onTap,
      child: Container(
        padding: widget.padding ?? EdgeInsets.all(0),
        alignment: Alignment.center,
        child: widget.text,
      ),
    );
  }
}

