import 'package:flutter/cupertino.dart';
import 'package:nui_core/model/enums/gradient_orientation.dart';

class NUILinearGradient extends LinearGradient{
  NUILinearGradient({@required this.colors, @required this.orientation}): super(colors:colors, begin:_alignmentFromOrientation(orientation).begin, end:_alignmentFromOrientation(orientation).end);

  static AlignmentPair _alignmentFromOrientation(GradientOrientation orientation){
    AlignmentPair align;
    switch(orientation){
      case GradientOrientation.B_T : {
        align = AlignmentPair(Alignment.bottomLeft, Alignment.topLeft);
        break;
      }
      case GradientOrientation.T_B : {
        align = AlignmentPair(Alignment.topLeft, Alignment.bottomLeft);
        break;
      }
      case GradientOrientation.L_R : {
        align = AlignmentPair(Alignment.topLeft, Alignment.topRight);
        break;
      }
      case GradientOrientation.R_L: {
        align = AlignmentPair(Alignment.topRight, Alignment.topLeft);
        break;
      }
      case GradientOrientation.TL_BR : {
        align = AlignmentPair(Alignment.topLeft, Alignment.bottomRight);
        break;
      }
      case GradientOrientation.TR_BL : {
        align = AlignmentPair(Alignment.topRight, Alignment.bottomLeft);
        break;
      }
      case GradientOrientation.BL_TR : {
        align = AlignmentPair(Alignment.bottomLeft, Alignment.topRight);
        break;
      }
      case GradientOrientation.BR_TL : {
        align = AlignmentPair(Alignment.bottomRight, Alignment.topLeft);
        break;
      }
      default :{
        align = AlignmentPair(Alignment.bottomLeft, Alignment.topLeft);
        break;
      }
    }
    return align;
  }

  final GradientOrientation orientation;
  final List<Color> colors;
}

class AlignmentPair{
  AlignmentPair(this.begin, this.end);
  final Alignment begin;
  final Alignment end;
}